@@ -22,87 +22,6 @@
  * DEALINGS IN THE SOFTWARE.
  */
 
-static bool object_is_client(struct pw_manager_object *o)
-{
-	return strcmp(o->type, PW_TYPE_INTERFACE_Client) == 0;
-}
-
-static bool object_is_module(struct pw_manager_object *o)
-{
-	return strcmp(o->type, PW_TYPE_INTERFACE_Module) == 0;
-}
-
-static bool object_is_card(struct pw_manager_object *o)
-{
-	const char *str;
-	return strcmp(o->type, PW_TYPE_INTERFACE_Device) == 0 &&
-		o->props != NULL &&
-		(str = pw_properties_get(o->props, PW_KEY_MEDIA_CLASS)) != NULL &&
-		strcmp(str, "Audio/Device") == 0;
-}
-
-static bool object_is_sink(struct pw_manager_object *o)
-{
-	const char *str;
-	return strcmp(o->type, PW_TYPE_INTERFACE_Node) == 0 &&
-		o->props != NULL &&
-		(str = pw_properties_get(o->props, PW_KEY_MEDIA_CLASS)) != NULL &&
-		(strcmp(str, "Audio/Sink") == 0 || strcmp(str, "Audio/Duplex") == 0);
-}
-
-static bool object_is_source(struct pw_manager_object *o)
-{
-	const char *str;
-	return strcmp(o->type, PW_TYPE_INTERFACE_Node) == 0 &&
-		o->props != NULL &&
-		(str = pw_properties_get(o->props, PW_KEY_MEDIA_CLASS)) != NULL &&
-		(strcmp(str, "Audio/Source") == 0 ||
-		 strcmp(str, "Audio/Duplex") == 0 ||
-		 strcmp(str, "Audio/Source/Virtual") == 0);
-}
-
-static bool object_is_monitor(struct pw_manager_object *o)
-{
-	const char *str;
-	return strcmp(o->type, PW_TYPE_INTERFACE_Node) == 0 &&
-		o->props != NULL &&
-		(str = pw_properties_get(o->props, PW_KEY_MEDIA_CLASS)) != NULL &&
-		(strcmp(str, "Audio/Sink") == 0);
-}
-
-static bool object_is_source_or_monitor(struct pw_manager_object *o)
-{
-	return object_is_source(o) || object_is_monitor(o);
-}
-
-static bool object_is_sink_input(struct pw_manager_object *o)
-{
-	const char *str;
-	return strcmp(o->type, PW_TYPE_INTERFACE_Node) == 0 &&
-		o->props != NULL &&
-		(str = pw_properties_get(o->props, PW_KEY_MEDIA_CLASS)) != NULL &&
-		strcmp(str, "Stream/Output/Audio") == 0;
-}
-
-static bool object_is_source_output(struct pw_manager_object *o)
-{
-	const char *str;
-	return strcmp(o->type, PW_TYPE_INTERFACE_Node) == 0 &&
-		o->props != NULL &&
-		(str = pw_properties_get(o->props, PW_KEY_MEDIA_CLASS)) != NULL &&
-		strcmp(str, "Stream/Input/Audio") == 0;
-}
-
-static bool object_is_recordable(struct pw_manager_object *o)
-{
-	return object_is_source(o) || object_is_sink(o) || object_is_sink_input(o);
-}
-
-static bool object_is_link(struct pw_manager_object *o)
-{
-	return strcmp(o->type, PW_TYPE_INTERFACE_Link) == 0;
-}
-
 struct selector {
 	bool (*type) (struct pw_manager_object *o);
 	uint32_t id;
@@ -119,7 +38,7 @@ static void select_best(struct selector *s, struct pw_manager_object *o)
 	int32_t prio = 0;
 
 	if (o->props &&
-	    (str = pw_properties_get(o->props, PW_KEY_PRIORITY_DRIVER)) != NULL) {
+	    (str = pw_properties_get(o->props, PW_KEY_PRIORITY_SESSION)) != NULL) {
 		prio = pw_properties_parse_int(str);
 		if (s->best == NULL || prio > s->score) {
 			s->best = o;
@@ -160,7 +79,7 @@ static struct pw_manager_object *find_linked(struct pw_manager *m, uint32_t obj_
 	uint32_t in_node, out_node;
 
 	spa_list_for_each(o, &m->object_list, link) {
-		if (o->props == NULL || !object_is_link(o))
+		if (o->props == NULL || !pw_manager_object_is_link(o))
 			continue;
 
 		if ((str = pw_properties_get(o->props, PW_KEY_LINK_OUTPUT_NODE)) == NULL)
@@ -171,12 +90,12 @@ static struct pw_manager_object *find_linked(struct pw_manager *m, uint32_t obj_
 		in_node = pw_properties_parse_int(str);
 
 		if (direction == PW_DIRECTION_OUTPUT && obj_id == out_node) {
-			struct selector sel = { .id = in_node, .type = object_is_sink, };
+			struct selector sel = { .id = in_node, .type = pw_manager_object_is_sink, };
 			if ((p = select_object(m, &sel)) != NULL)
 				return p;
 		}
 		if (direction == PW_DIRECTION_INPUT && obj_id == in_node) {
-			struct selector sel = { .id = out_node, .type = object_is_recordable, };
+			struct selector sel = { .id = out_node, .type = pw_manager_object_is_recordable, };
 			if ((p = select_object(m, &sel)) != NULL)
 				return p;
 		}
@@ -553,3 +472,105 @@ static struct spa_dict *collect_props(struct spa_pod *info, struct spa_dict *dic
 	dict->n_items = n;
 	return dict;
 }
+
+struct transport_codec_info {
+	enum spa_bluetooth_audio_codec id;
+	const char *description;
+};
+
+static uint32_t collect_transport_codec_info(struct pw_manager_object *card,
+		struct transport_codec_info *codecs, uint32_t max_codecs, uint32_t *active)
+{
+	struct pw_manager_param *p;
+	uint32_t n_codecs = 0;
+
+	*active = SPA_ID_INVALID;
+
+	if (card == NULL)
+		return 0;
+
+	spa_list_for_each(p, &card->param_list, link) {
+		uint32_t iid;
+		const struct spa_pod_choice *type;
+		const struct spa_pod_struct *labels;
+		struct spa_pod_parser prs;
+		struct spa_pod_frame f;
+		int32_t *id;
+		bool first;
+
+		if (p->id != SPA_PARAM_PropInfo)
+			continue;
+
+		if (spa_pod_parse_object(p->param,
+						SPA_TYPE_OBJECT_PropInfo, NULL,
+						SPA_PROP_INFO_id, SPA_POD_Id(&iid),
+						SPA_PROP_INFO_type, SPA_POD_PodChoice(&type),
+						SPA_PROP_INFO_labels, SPA_POD_PodStruct(&labels)) < 0)
+			continue;
+
+		if (iid != SPA_PROP_bluetoothAudioCodec)
+			continue;
+
+		if (SPA_POD_CHOICE_TYPE(type) != SPA_CHOICE_Enum ||
+				SPA_POD_TYPE(SPA_POD_CHOICE_CHILD(type)) != SPA_TYPE_Int)
+			continue;
+
+		/*
+		 * XXX: PropInfo currently uses Int, not Id, in type and labels.
+		 */
+
+		/* Codec name list */
+		first = true;
+		SPA_POD_CHOICE_FOREACH(type, id) {
+			if (first) {
+				/* Skip default */
+				first = false;
+				continue;
+			}
+			if (n_codecs >= max_codecs)
+				break;
+			codecs[n_codecs++].id = *id;
+		}
+
+		/* Codec description list */
+		spa_pod_parser_pod(&prs, (struct spa_pod *)labels);
+		if (spa_pod_parser_push_struct(&prs, &f) < 0)
+			continue;
+
+		while (1) {
+			int32_t id;
+			const char *desc;
+			uint32_t j;
+
+			if (spa_pod_parser_get_int(&prs, &id) < 0 ||
+					spa_pod_parser_get_string(&prs, &desc) < 0)
+				break;
+
+			for (j = 0; j < n_codecs; ++j) {
+				if (codecs[j].id == (uint32_t)id)
+					codecs[j].description = desc;
+			}
+		}
+	}
+
+	/* Active codec */
+	spa_list_for_each(p, &card->param_list, link) {
+		uint32_t j;
+		uint32_t id;
+
+		if (p->id != SPA_PARAM_Props)
+			continue;
+
+		if (spa_pod_parse_object(p->param,
+						SPA_TYPE_OBJECT_Props, NULL,
+						SPA_PROP_bluetoothAudioCodec, SPA_POD_Id(&id)) < 0)
+			continue;
+
+		for (j = 0; j < n_codecs; ++j) {
+			if (codecs[j].id == id)
+				*active = j;
+		}
+	}
+
+	return n_codecs;
+}
