@@ -22,47 +22,7 @@
  * DEALINGS IN THE SOFTWARE.
  */
 
-#define RATE_MAX	(48000u*8u)
-#define CHANNELS_MAX	(64u)
-
-enum sample_format {
-	SAMPLE_U8,
-	SAMPLE_ALAW,
-	SAMPLE_ULAW,
-	SAMPLE_S16LE,
-	SAMPLE_S16BE,
-	SAMPLE_FLOAT32LE,
-	SAMPLE_FLOAT32BE,
-	SAMPLE_S32LE,
-	SAMPLE_S32BE,
-	SAMPLE_S24LE,
-	SAMPLE_S24BE,
-	SAMPLE_S24_32LE,
-	SAMPLE_S24_32BE,
-	SAMPLE_MAX,
-	SAMPLE_INVALID = -1
-};
-
-#if __BYTE_ORDER == __BIG_ENDIAN
-#define SAMPLE_S16NE		SAMPLE_S16BE
-#define SAMPLE_FLOAT32NE	SAMPLE_FLOAT32BE
-#define SAMPLE_S32NE		SAMPLE_S32BE
-#define	SAMPLE_S24NE		SAMPLE_S24BE
-#define SAMPLE_S24_32NE		SAMPLE_S24_32BE
-#elif __BYTE_ORDER == __LITTLE_ENDIAN
-#define SAMPLE_S16NE		SAMPLE_S16LE
-#define SAMPLE_FLOAT32NE	SAMPLE_FLOAT32LE
-#define SAMPLE_S32NE		SAMPLE_S32LE
-#define	SAMPLE_S24NE		SAMPLE_S24LE
-#define SAMPLE_S24_32NE		SAMPLE_S24_32LE
-#endif
-
-struct format {
-	uint32_t pa;
-	uint32_t id;
-	const char *name;
-	uint32_t size;
-};
+#include "format.h"
 
 static const struct format audio_formats[] = {
 	[SAMPLE_U8] = { SAMPLE_U8, SPA_AUDIO_FORMAT_U8, "u8", 1 },
@@ -79,23 +39,36 @@ static const struct format audio_formats[] = {
 	[SAMPLE_S24_32LE] = { SAMPLE_S24_32LE, SPA_AUDIO_FORMAT_S24_32_LE, "s24-32le", 4 },
 	[SAMPLE_S24_32BE] = { SAMPLE_S24_32BE, SPA_AUDIO_FORMAT_S24_32_BE, "s24-32be", 4 },
 
+#if __BYTE_ORDER == __BIG_ENDIAN
+	{ SAMPLE_S16BE, SPA_AUDIO_FORMAT_S16_BE, "s16ne", 2 },
+	{ SAMPLE_FLOAT32BE, SPA_AUDIO_FORMAT_F32_BE, "float32ne", 4 },
+	{ SAMPLE_S32BE, SPA_AUDIO_FORMAT_S32_BE, "s32ne", 4 },
+	{ SAMPLE_S24BE, SPA_AUDIO_FORMAT_S24_BE, "s24ne", 3 },
+	{ SAMPLE_S24_32BE, SPA_AUDIO_FORMAT_S24_32_BE, "s24-32ne", 4 },
+#elif __BYTE_ORDER == __LITTLE_ENDIAN
+	{ SAMPLE_S16LE, SPA_AUDIO_FORMAT_S16_LE, "s16ne", 2 },
+	{ SAMPLE_FLOAT32LE, SPA_AUDIO_FORMAT_F32_LE, "float32ne", 4 },
+	{ SAMPLE_S32LE, SPA_AUDIO_FORMAT_S32_LE, "s32ne", 4 },
+	{ SAMPLE_S24LE, SPA_AUDIO_FORMAT_S24_LE, "s24ne", 3 },
+	{ SAMPLE_S24_32LE, SPA_AUDIO_FORMAT_S24_32_LE, "s24-32ne", 4 },
+#endif
 	/* planar formats, we just report them as inteleaved */
-	{ SAMPLE_U8, SPA_AUDIO_FORMAT_U8P, "u8", 1 },
-	{ SAMPLE_S16NE, SPA_AUDIO_FORMAT_S16P, "s16", 2 },
-	{ SAMPLE_S24_32NE, SPA_AUDIO_FORMAT_S24_32P, "s24-32", 4 },
-	{ SAMPLE_S32NE, SPA_AUDIO_FORMAT_S32P, "s32", 4 },
-	{ SAMPLE_S24NE, SPA_AUDIO_FORMAT_S24P, "s24", 3 },
-	{ SAMPLE_FLOAT32NE, SPA_AUDIO_FORMAT_F32P, "float32", 4 },
+	{ SAMPLE_U8, SPA_AUDIO_FORMAT_U8P, "u8ne", 1 },
+	{ SAMPLE_S16NE, SPA_AUDIO_FORMAT_S16P, "s16ne", 2 },
+	{ SAMPLE_S24_32NE, SPA_AUDIO_FORMAT_S24_32P, "s24-32ne", 4 },
+	{ SAMPLE_S32NE, SPA_AUDIO_FORMAT_S32P, "s32ne", 4 },
+	{ SAMPLE_S24NE, SPA_AUDIO_FORMAT_S24P, "s24ne", 3 },
+	{ SAMPLE_FLOAT32NE, SPA_AUDIO_FORMAT_F32P, "float32ne", 4 },
 };
 
-static inline uint32_t format_pa2id(enum sample_format format)
+uint32_t format_pa2id(enum sample_format format)
 {
 	if (format < 0 || format >= SAMPLE_MAX)
 		return SPA_AUDIO_FORMAT_UNKNOWN;
 	return audio_formats[format].id;
 }
 
-static inline const char *format_id2name(uint32_t format)
+const char *format_id2name(uint32_t format)
 {
 	int i;
 	for (i = 0; spa_type_audio_format[i].name; i++) {
@@ -104,18 +77,28 @@ static inline const char *format_id2name(uint32_t format)
 	}
 	return "UNKNOWN";
 }
+uint32_t format_name2id(const char *name)
+{
+	int i;
+	for (i = 0; spa_type_audio_format[i].name; i++) {
+		if (strcmp(name, spa_debug_type_short_name(spa_type_audio_format[i].name)) == 0)
+			return spa_type_audio_format[i].type;
+	}
+	return SPA_AUDIO_CHANNEL_UNKNOWN;
+}
 
-static inline uint32_t format_paname2id(const char *name, size_t size)
+uint32_t format_paname2id(const char *name, size_t size)
 {
 	size_t i;
-	for (i = 0; i < SAMPLE_MAX; i++) {
-		if (strncmp(name, audio_formats[i].name, size) == 0)
+	for (i = 0; i < SPA_N_ELEMENTS(audio_formats); i++) {
+		if (audio_formats[i].name != NULL &&
+		    strncmp(name, audio_formats[i].name, size) == 0)
 			return audio_formats[i].id;
 	}
 	return SPA_AUDIO_FORMAT_UNKNOWN;
 }
 
-static inline enum sample_format format_id2pa(uint32_t id)
+enum sample_format format_id2pa(uint32_t id)
 {
 	size_t i;
 	for (i = 0; i < SPA_N_ELEMENTS(audio_formats); i++) {
@@ -125,33 +108,18 @@ static inline enum sample_format format_id2pa(uint32_t id)
 	return SAMPLE_INVALID;
 }
 
-static inline const char *format_id2paname(uint32_t id)
+const char *format_id2paname(uint32_t id)
 {
 	size_t i;
 	for (i = 0; i < SPA_N_ELEMENTS(audio_formats); i++) {
-		if (id == audio_formats[i].id)
+		if (id == audio_formats[i].id &&
+		    audio_formats[i].name != NULL)
 			return audio_formats[i].name;
 	}
 	return "invalid";
 }
 
-struct sample_spec {
-	uint32_t format;
-	uint32_t rate;
-	uint8_t channels;
-};
-#define SAMPLE_SPEC_INIT	(struct sample_spec) {				\
-					.format = SPA_AUDIO_FORMAT_UNKNOWN,	\
-					.rate = 0,				\
-					.channels = 0,				\
-				}
-#define SAMPLE_SPEC_DEFAULT	(struct sample_spec) {			\
-					.format = SPA_AUDIO_FORMAT_F32,	\
-					.rate = 44100,			\
-					.channels = 2,			\
-				}
-
-static inline uint32_t sample_spec_frame_size(const struct sample_spec *ss)
+uint32_t sample_spec_frame_size(const struct sample_spec *ss)
 {
 	switch (ss->format) {
 	case SPA_AUDIO_FORMAT_U8:
@@ -179,81 +147,13 @@ static inline uint32_t sample_spec_frame_size(const struct sample_spec *ss)
 	}
 }
 
-static inline bool sample_spec_valid(const struct sample_spec *ss)
+bool sample_spec_valid(const struct sample_spec *ss)
 {
 	return (sample_spec_frame_size(ss) > 0 &&
 	    ss->rate > 0 && ss->rate <= RATE_MAX &&
 	    ss->channels > 0 && ss->channels <= CHANNELS_MAX);
 }
 
-enum channel_position {
-	CHANNEL_POSITION_INVALID = -1,
-	CHANNEL_POSITION_MONO = 0,
-	CHANNEL_POSITION_FRONT_LEFT,
-	CHANNEL_POSITION_FRONT_RIGHT,
-	CHANNEL_POSITION_FRONT_CENTER,
-
-	CHANNEL_POSITION_REAR_CENTER,
-	CHANNEL_POSITION_REAR_LEFT,
-	CHANNEL_POSITION_REAR_RIGHT,
-
-	CHANNEL_POSITION_LFE,
-	CHANNEL_POSITION_FRONT_LEFT_OF_CENTER,
-	CHANNEL_POSITION_FRONT_RIGHT_OF_CENTER,
-
-	CHANNEL_POSITION_SIDE_LEFT,
-	CHANNEL_POSITION_SIDE_RIGHT,
-	CHANNEL_POSITION_AUX0,
-	CHANNEL_POSITION_AUX1,
-	CHANNEL_POSITION_AUX2,
-	CHANNEL_POSITION_AUX3,
-	CHANNEL_POSITION_AUX4,
-	CHANNEL_POSITION_AUX5,
-	CHANNEL_POSITION_AUX6,
-	CHANNEL_POSITION_AUX7,
-	CHANNEL_POSITION_AUX8,
-	CHANNEL_POSITION_AUX9,
-	CHANNEL_POSITION_AUX10,
-	CHANNEL_POSITION_AUX11,
-	CHANNEL_POSITION_AUX12,
-	CHANNEL_POSITION_AUX13,
-	CHANNEL_POSITION_AUX14,
-	CHANNEL_POSITION_AUX15,
-	CHANNEL_POSITION_AUX16,
-	CHANNEL_POSITION_AUX17,
-	CHANNEL_POSITION_AUX18,
-	CHANNEL_POSITION_AUX19,
-	CHANNEL_POSITION_AUX20,
-	CHANNEL_POSITION_AUX21,
-	CHANNEL_POSITION_AUX22,
-	CHANNEL_POSITION_AUX23,
-	CHANNEL_POSITION_AUX24,
-	CHANNEL_POSITION_AUX25,
-	CHANNEL_POSITION_AUX26,
-	CHANNEL_POSITION_AUX27,
-	CHANNEL_POSITION_AUX28,
-	CHANNEL_POSITION_AUX29,
-	CHANNEL_POSITION_AUX30,
-	CHANNEL_POSITION_AUX31,
-
-	CHANNEL_POSITION_TOP_CENTER,
-
-	CHANNEL_POSITION_TOP_FRONT_LEFT,
-	CHANNEL_POSITION_TOP_FRONT_RIGHT,
-	CHANNEL_POSITION_TOP_FRONT_CENTER,
-
-	CHANNEL_POSITION_TOP_REAR_LEFT,
-	CHANNEL_POSITION_TOP_REAR_RIGHT,
-	CHANNEL_POSITION_TOP_REAR_CENTER,
-
-	CHANNEL_POSITION_MAX
-};
-
-struct channel {
-	uint32_t channel;
-	const char *name;
-};
-
 static const struct channel audio_channels[] = {
 	[CHANNEL_POSITION_MONO] = { SPA_AUDIO_CHANNEL_MONO, "mono", },
 
@@ -316,28 +216,14 @@ static const struct channel audio_channels[] = {
 	[CHANNEL_POSITION_TOP_REAR_CENTER] = { SPA_AUDIO_CHANNEL_TRC, "top-rear-center", },
 };
 
-struct channel_map {
-	uint8_t channels;
-	uint32_t map[CHANNELS_MAX];
-};
-
-#define CHANNEL_MAP_INIT	(struct channel_map) {				\
-					.channels = 0,				\
-				}
-#define CHANNEL_MAP_DEFAULT	(struct channel_map) {				\
-					.channels = 2,                          \
-					.map[0] = SPA_AUDIO_CHANNEL_FL,		\
-					.map[1] = SPA_AUDIO_CHANNEL_FR,		\
-				}
-
-static inline uint32_t channel_pa2id(enum channel_position channel)
+uint32_t channel_pa2id(enum channel_position channel)
 {
         if (channel < 0 || (size_t)channel >= SPA_N_ELEMENTS(audio_channels))
                 return SPA_AUDIO_CHANNEL_UNKNOWN;
         return audio_channels[channel].channel;
 }
 
-static inline const char *channel_id2name(uint32_t channel)
+const char *channel_id2name(uint32_t channel)
 {
 	int i;
 	for (i = 0; spa_type_audio_channel[i].name; i++) {
@@ -347,7 +233,7 @@ static inline const char *channel_id2name(uint32_t channel)
 	return "UNK";
 }
 
-static inline uint32_t channel_name2id(const char *name)
+uint32_t channel_name2id(const char *name)
 {
 	int i;
 	for (i = 0; spa_type_audio_channel[i].name; i++) {
@@ -357,27 +243,28 @@ static inline uint32_t channel_name2id(const char *name)
 	return SPA_AUDIO_CHANNEL_UNKNOWN;
 }
 
-static inline enum channel_position channel_id2pa(uint32_t id, uint32_t *aux)
+enum channel_position channel_id2pa(uint32_t id, uint32_t *aux)
 {
 	size_t i;
 	for (i = 0; i < SPA_N_ELEMENTS(audio_channels); i++) {
 		if (id == audio_channels[i].channel)
 			return i;
 	}
-	return CHANNEL_POSITION_AUX0 + (*aux)++;
+	return CHANNEL_POSITION_AUX0 + ((*aux)++ & 31);
 }
 
-static inline const char *channel_id2paname(uint32_t id, uint32_t *aux)
+const char *channel_id2paname(uint32_t id, uint32_t *aux)
 {
 	size_t i;
 	for (i = 0; i < SPA_N_ELEMENTS(audio_channels); i++) {
-		if (id == audio_channels[i].channel)
+		if (id == audio_channels[i].channel &&
+		    audio_channels[i].name != NULL)
 			return audio_channels[i].name;
 	}
-	return audio_channels[CHANNEL_POSITION_AUX0 + (*aux)++].name;
+	return audio_channels[CHANNEL_POSITION_AUX0 + ((*aux)++ & 31)].name;
 }
 
-static inline uint32_t channel_paname2id(const char *name, size_t size)
+uint32_t channel_paname2id(const char *name, size_t size)
 {
 	size_t i;
 	for (i = 0; i < SPA_N_ELEMENTS(audio_channels); i++) {
@@ -388,14 +275,14 @@ static inline uint32_t channel_paname2id(const char *name, size_t size)
 }
 
 
-static inline void channel_map_to_positions(const struct channel_map *map, uint32_t *pos)
+void channel_map_to_positions(const struct channel_map *map, uint32_t *pos)
 {
 	int i;
 	for (i = 0; i < map->channels; i++)
 		pos[i] = map->map[i];
 }
 
-static inline void channel_map_parse(const char *str, struct channel_map *map)
+void channel_map_parse(const char *str, struct channel_map *map)
 {
 	const char *p = str;
 	size_t len;
@@ -472,7 +359,7 @@ static inline void channel_map_parse(const char *str, struct channel_map *map)
 	}
 }
 
-static inline bool channel_map_valid(const struct channel_map *map)
+bool channel_map_valid(const struct channel_map *map)
 {
 	uint8_t i;
 	if (map->channels == 0 || map->channels > CHANNELS_MAX)
@@ -484,20 +371,6 @@ static inline bool channel_map_valid(const struct channel_map *map)
 }
 
 
-enum encoding {
-	ENCODING_ANY,
-	ENCODING_PCM,
-	ENCODING_AC3_IEC61937,
-	ENCODING_EAC3_IEC61937,
-	ENCODING_MPEG_IEC61937,
-	ENCODING_DTS_IEC61937,
-	ENCODING_MPEG2_AAC_IEC61937,
-	ENCODING_TRUEHD_IEC61937,
-	ENCODING_DTSHD_IEC61937,
-	ENCODING_MAX,
-	ENCODING_INVALID = -1,
-};
-
 static const char *encoding_names[] = {
 	[ENCODING_ANY] = "ANY",
 	[ENCODING_PCM] = "PCM",
@@ -510,9 +383,10 @@ static const char *encoding_names[] = {
 	[ENCODING_DTSHD_IEC61937] = "DTSHD-IEC61937",
 };
 
-static inline const char *format_encoding2name(enum encoding enc)
+const char *format_encoding2name(enum encoding enc)
 {
-	if (enc >= 0 && enc < (int)SPA_N_ELEMENTS(encoding_names))
+	if (enc >= 0 && enc < (int)SPA_N_ELEMENTS(encoding_names) &&
+	    encoding_names[enc] != NULL)
 		return encoding_names[enc];
 	return "INVALID";
 }
@@ -534,7 +408,8 @@ static int format_parse_param(const struct spa_pod *param, struct sample_spec *s
 	struct spa_audio_info info = { 0 };
 	uint32_t i;
 
-        spa_format_parse(param, &info.media_type, &info.media_subtype);
+	if (spa_format_parse(param, &info.media_type, &info.media_subtype) < 0)
+		return -ENOTSUP;
 
 	if (info.media_type != SPA_MEDIA_TYPE_audio ||
 	    info.media_subtype != SPA_MEDIA_SUBTYPE_raw ||
