@@ -23,43 +23,23 @@
  * DEALINGS IN THE SOFTWARE.
  */
 
-struct module;
+#include "module.h"
 
-struct module_info {
-	const char *name;
-	struct module *(*create) (struct impl *impl, const char *args);
-};
+static int module_unload(struct client *client, struct module *module);
 
-struct module_events {
-#define VERSION_MODULE_EVENTS	0
-	uint32_t version;
+static void on_module_unload(void *obj, void *data, int res, uint32_t id)
+{
+	struct module *module = obj;
+	module_unload(NULL, module);
+}
 
-	void (*loaded) (void *data, int res);
-};
+void module_schedule_unload(struct module *module)
+{
+	struct impl *impl = module->impl;
+	pw_work_queue_add(impl->work_queue, module, 0, on_module_unload, impl);
+}
 
-#define module_emit_loaded(m,r) spa_hook_list_call(&m->hooks, struct module_events, loaded, 0, r)
-
-struct module_methods {
-#define VERSION_MODULE_METHODS	0
-	uint32_t version;
-
-	int (*load) (struct client *client, struct module *module);
-	int (*unload) (struct client *client, struct module *module);
-};
-
-struct module {
-	uint32_t idx;
-	const char *name;
-	const char *args;
-	struct pw_properties *props;
-	struct spa_list link;           /**< link in client modules */
-	struct impl *impl;
-	const struct module_methods *methods;
-	struct spa_hook_list hooks;
-	void *user_data;
-};
-
-static struct module *module_new(struct impl *impl, const struct module_methods *methods, size_t user_data)
+struct module *module_new(struct impl *impl, const struct module_methods *methods, size_t user_data)
 {
 	struct module *module;
 
@@ -87,6 +67,8 @@ static int module_load(struct client *client, struct module *module)
 	pw_log_info("load module id:%u name:%s", module->idx, module->name);
 	if (module->methods->load == NULL)
 		return -ENOTSUP;
+	/* subscription event is sent when the module does a
+	 * module_emit_loaded() */
 	return module->methods->load(client, module);
 }
 
@@ -96,6 +78,7 @@ static void module_free(struct module *module)
 	if (module->idx != SPA_ID_INVALID)
 		pw_map_remove(&impl->modules, module->idx & INDEX_MASK);
 
+	pw_work_queue_cancel(impl->work_queue, module, SPA_ID_INVALID);
 	free((char*)module->name);
 	free((char*)module->args);
 	if (module->props)
@@ -105,19 +88,30 @@ static void module_free(struct module *module)
 
 static int module_unload(struct client *client, struct module *module)
 {
+	struct impl *impl = module->impl;
+	uint32_t module_idx = module->idx;
 	int res = 0;
 
+	/* Note that client can be NULL (when the module is being unloaded
+	 * internally and not by a client request */
+
 	pw_log_info("unload module id:%u name:%s", module->idx, module->name);
 
 	if (module->methods->unload)
 		res = module->methods->unload(client, module);
 
 	module_free(module);
+
+	broadcast_subscribe_event(impl,
+			SUBSCRIPTION_MASK_MODULE,
+			SUBSCRIPTION_EVENT_REMOVE | SUBSCRIPTION_EVENT_MODULE,
+			module_idx);
+
 	return res;
 }
 
 /** utils */
-static void add_props(struct pw_properties *props, const char *str)
+void module_args_add_props(struct pw_properties *props, const char *str)
 {
 	char *s = strdup(str), *p = s, *e, f;
 	const char *k, *v;
@@ -151,10 +145,73 @@ static void add_props(struct pw_properties *props, const char *str)
 	free(s);
 }
 
-#include "module-null-sink.c"
+int module_args_to_audioinfo(struct impl *impl, struct pw_properties *props, struct spa_audio_info_raw *info)
+{
+	const char *str;
+	uint32_t i;
+
+	/* We don't use any incoming format setting and use our native format */
+	spa_zero(*info);
+	info->format = SPA_AUDIO_FORMAT_F32P;
+
+	if ((str = pw_properties_get(props, "channels")) != NULL) {
+		info->channels = pw_properties_parse_int(str);
+		if (info->channels == 0 || info->channels > SPA_AUDIO_MAX_CHANNELS) {
+			pw_log_error("invalid channels '%s'", str);
+			return -EINVAL;
+		}
+		pw_properties_set(props, "channels", NULL);
+	}
+	if ((str = pw_properties_get(props, "channel_map")) != NULL) {
+		struct channel_map map;
+
+		channel_map_parse(str, &map);
+		if (map.channels == 0 || map.channels > SPA_AUDIO_MAX_CHANNELS) {
+			pw_log_error("invalid channel_map '%s'", str);
+			return -EINVAL;
+		}
+		if (info->channels == 0)
+			info->channels = map.channels;
+		if (info->channels != map.channels) {
+			pw_log_error("Mismatched channel map");
+			return -EINVAL;
+		}
+		channel_map_to_positions(&map, info->position);
+		pw_properties_set(props, "channel_map", NULL);
+	} else {
+		if (info->channels == 0)
+			info->channels = impl->defs.sample_spec.channels;
+
+		if (info->channels == impl->defs.channel_map.channels) {
+			channel_map_to_positions(&impl->defs.channel_map, info->position);
+		} else if (info->channels == 1) {
+			info->position[0] = SPA_AUDIO_CHANNEL_MONO;
+		} else if (info->channels == 2) {
+			info->position[0] = SPA_AUDIO_CHANNEL_FL;
+			info->position[1] = SPA_AUDIO_CHANNEL_FR;
+		} else {
+			/* FIXME add more mappings */
+			for (i = 0; i < info->channels; i++)
+				info->position[i] = SPA_AUDIO_CHANNEL_UNKNOWN;
+		}
+	}
+
+	if ((str = pw_properties_get(props, "rate")) != NULL) {
+		info->rate = pw_properties_parse_int(str);
+		pw_properties_set(props, "rate", NULL);
+	} else {
+		info->rate = 0;
+	}
+	return 0;
+}
+
+#include "modules/registry.h"
 
 static const struct module_info module_list[] = {
+	{ "module-loopback", create_module_loopback, },
 	{ "module-null-sink", create_module_null_sink, },
+	{ "module-native-protocol-tcp", create_module_native_protocol_tcp, },
+	{ "module-simple-protocol-tcp", create_module_simple_protocol_tcp, },
 	{ NULL, }
 };
 
@@ -176,7 +233,7 @@ static struct module *create_module(struct client *client, const char *name, con
 
 	info = find_module_info(name);
 	if (info == NULL) {
-		errno = -ENOENT;
+		errno = ENOENT;
 		return NULL;
 	}
 	module = info->create(impl, args);
@@ -189,7 +246,7 @@ static struct module *create_module(struct client *client, const char *name, con
 		return NULL;
 	}
 	module->name = strdup(name);
-	module->args = strdup(args);
+	module->args = args ? strdup(args) : NULL;
 	module->idx |= MODULE_FLAG;
 	return module;
 }
