@@ -30,6 +30,7 @@ extern "C" {
 #endif
 
 #include <spa/support/plugin.h>
+#include <spa/support/i18n.h>
 
 #include <pipewire/array.h>
 #include <pipewire/client.h>
@@ -141,6 +142,9 @@ bool pw_in_valgrind(void);
 enum pw_direction
 pw_direction_reverse(enum pw_direction direction);
 
+int pw_set_domain(const char *domain);
+const char *pw_get_domain(void);
+
 uint32_t pw_get_support(struct spa_support *support, uint32_t max_support);
 
 struct spa_handle *pw_load_spa_handle(const char *lib,
@@ -151,6 +155,12 @@ struct spa_handle *pw_load_spa_handle(const char *lib,
 
 int pw_unload_spa_handle(struct spa_handle *handle);
 
+const char *pw_gettext(const char *msgid);
+const char *pw_ngettext(const char *msgid, const char *msgid_plural, unsigned long int n);
+
+#define _(String)	(pw_gettext(String))
+#define N_(String)	(String)
+
 #ifdef __cplusplus
 }
 #endif
